

from biogeme import *
from headers import *
from loglikelihood import *
from statistics import *


# Parameters to be estimated
# Arguments:
#   1  Name for report. Typically, the same as the variable
#   2  Starting value
#   3  Lower bound
#   4  Upper bound
#   5  0: estimate the parameter, 1: keep it fixed

ASC_TRAIN = Beta('ASC_TRAIN',0,-10,10,0,'ASC_TRAIN' )
B_TIME = Beta('B_TIME',0,-10,10,0,'B_TIME' )
B_TIME_S = Beta('B_TIME_S',1,-10,10,0,'B_TIME_S' )
B_COST = Beta('B_COST',0,-10,10,0,'B_COST' )
B_COST_S = Beta('B_COST_S',1,-10,10,0,'B_TIME_S' )
ASC_BUS = Beta('ASC_BUS',0,-10,10,0,'ASC_BUS' )
ASC_CAR = Beta('ASC_CAR',0,-10,10,0,'ASC_CAR' )
ASC_WALK = Beta('ASC_WALK',0,-10,10,1,'ASC_WALK' )
ASC_BIKE = Beta('ASC_BIKE',0,-10,10,0,'ASC_BIKE' )

# Define a random parameter, log normally distirbuted, designed to be used
# for Monte-Carlo simulation
B_TIME_RND = -exp(B_TIME + B_TIME_S * bioDraws('B_TIME_RND'))

# Utility functions

# For numerical reasons, it is good practice to scale the data to
# that the values of the parameters are around 1.0. 
# A previous estimation with the unscaled data has generated
# parameters around -0.01 for both cost and time. Therefore, time and
# cost are multipled my 0.01.

TRAIN_TT_SCALED = DefineVariable('TRAIN_TT_SCALED', TotalTimeTrain / 100.0)
TRAIN_CO_SCALED = DefineVariable('TRAIN_CO_SCALED', FareTrain / 100)
BUS_TT_SCALED = DefineVariable('BUS_TT_SCALED', TotalTimeBus / 100.0)
BUS_CO_SCALED = DefineVariable('BUS_CO_SCALED', FareBus / 100)
CAR_TT_SCALED = DefineVariable('CAR_TT_SCALED', TimeCar / 100.0)
WALK_TT_SCALED = DefineVariable('WALK_TT_SCALED', TimeWalk / 100.0)
BIKE_TT_SCALED = DefineVariable('BIKE_TT_SCALED', TimeBike / 100.0)

V1 = ASC_TRAIN + B_TIME_RND * TRAIN_TT_SCALED + B_COST * TRAIN_CO_SCALED
V2 = ASC_BUS + B_TIME_RND * BUS_TT_SCALED + B_COST * BUS_CO_SCALED
V3 = ASC_CAR + B_TIME_RND * CAR_TT_SCALED
V4 = ASC_WALK + B_TIME_RND * WALK_TT_SCALED 
V5 = ASC_BIKE + B_TIME_RND * BIKE_TT_SCALED 

# Associate utility functions with the numbering of alternatives
V = {1: V1,
     2: V2,
     3: V3,
     4: V4,
     5: V5}

# Associate the availability conditions with the alternatives

av = {1: ModeAvailableTrain,
      2: ModeAvailableBus,
      3: ModeAvailableCar,
      4: ModeAvailableWalk,
      5: ModeAvailableBike}

# The choice model is a logit, with availability conditions
prob = bioLogit(V,av,Choice)
l = mixedloglikelihood(prob)

# Defines an itertor on the data
rowIterator('obsIter') 

# Define the likelihood function for the estimation
BIOGEME_OBJECT.ESTIMATE = Sum(l,'obsIter')

# Statistics

nullLoglikelihood(av,'obsIter')
choiceSet = [1,2,3,4,5]
cteLoglikelihood(choiceSet,Choice,'obsIter')
availabilityStatistics(av,'obsIter')

BIOGEME_OBJECT.PARAMETERS['NbrOfDraws'] = "500"

BIOGEME_OBJECT.PARAMETERS['optimizationAlgorithm'] = "BIO"
BIOGEME_OBJECT.DRAWS = { 'B_TIME_RND': 'NORMAL' }
BIOGEME_OBJECT.FORMULAS['Train utility'] = V1
BIOGEME_OBJECT.FORMULAS['Bus utility'] = V2
BIOGEME_OBJECT.FORMULAS['Car utility'] = V3
BIOGEME_OBJECT.FORMULAS['Walk utility'] = V4
BIOGEME_OBJECT.FORMULAS['Bike utility'] = V5
