### Mixed Logit model estimation
install.packages("maxLik")
library(maxLik)

### データファイルの読み込み
Data <- read.csv("ensyu.csv",header=T)
## データ数:Dataの行数を数える
hh <- nrow(Data)

## パラメータの初期値の設定
#推定するパラメータの初期値0を生成
b0 <- numeric(7) 

##### 乱数の発生 #####

## 乱数の発生回数 
R <- 100

## 乱数の発生-ハルトン数列関数による方法 
#### ハルトン数列関数 
halton <- function(n,x) {
  no <- 1:n
  hal <- numeric(n)
  i <- 1
  while (n >= x^i) {
    i <- i + 1
  }
  for (j in i:1 ) {
    hal <- hal + ( no %/% (x^(j-1)) ) * (1/ x^j)
    no <- no %% x^(j-1)
  }
  hal
}

#0~1の区間をとるハルトン数列を，qnorm(標準正規分布の確率密度関数の逆関数)によって標準正規分布の準乱数数列に変換
#matrix(ベクトル，行数，数列)
rand1 <- matrix(qnorm(halton(hh*R,2)),hh,R)
rand2 <- matrix(qnorm(halton(hh*R,3)),hh,R)

##### Logit model の対数尤度関数の定義 #####

fr <- function(x) {
  ### パラメータの宣言：
  ## 定数項
  b1 <- x[1]
  b2 <- x[2]
  b3 <- x[3]
  b4 <- x[4]

  ## 目的地までの所要時間
  dmu1 <- x[5] #平均μ
  dsigma1 <- x[6] #標準偏差σ 

  ## 料金
  f1 <- x[7] #料金パラメータは固定してみる

  ## シミュレーション尤度のための変数を宣言
  SimLL = 0

  ### 今回用いる交通手段は以下の5つ．
  ## 鉄道(train)
  ## バス(bus)
  ## 自動車(car)
  ## 自転車(bike)
  ## 徒歩(walk)
  
  ##R回シミュレーションを行うループ
  #後で平均取るためにRで対数尤度smillを割る
  for (i in 1:R) {
  
    ## 対数尤度のための変数を宣言
    LL = 0
  
    ##乱数からパラメータをシミュレーションで発生 
    #dum1，fmu1は平均，dsigma1，fsigma1は標準偏差,rand1[,i]，rand2[,i]はハルトン数列法で生成した標準正規分布の順乱数
    #d1，f1は所要時間，費用(サービスレベル変数)のパラメータで，パラメータ自身が確率的に変動するランダム係数となっている
    #[,i]はその行列のi行目を取り出すという意味
    d1 <- dmu1 + dsigma1 * rand1[,i]
    #f1 <- fmu1 + fsigma1 * rand2[,i]

    
    #Data$代替手段生成可否trainは「主な移動手段がtrainなら1，そうでなければ0」という意味．(csvファイルに1が格納されてる)
    #$でDataとして読み込んだcsvファイル内の各列を固定
    #exp(V)の式を各々の選択肢について作っている↓
    #matrix(1,nrow =hh,ncol=1)は「hh(400)行1列」で1が格納された行列
    #100で割ってる理由はサービスレベル変数(時間，料金)のおおよその単位をそろえて推定結果を安定させるため
    #おのおの100分，100円単位にしている
                                                      # 時間                     # 料金                    # 定数項
    train  <- Data$代替手段生成可否train*exp(d1*Data$総所要時間train/100 +f1*Data$費用train/100 + b1*matrix(1,nrow =hh,ncol=1))
    bus    <- Data$代替手段生成可否bus  *exp(d1*Data$総所要時間bus/100   +f1*Data$費用bus/100   + b2*matrix(1,nrow =hh,ncol=1))
    car    <- Data$代替手段生成可否car  *exp(d1*Data$所要時間car/100                        + b3*matrix(1,nrow =hh,ncol=1))
    bike   <- Data$代替手段生成可否bike *exp(d1*Data$所要時間bike/100                       + b4*matrix(1,nrow =hh,ncol=1))
    walk   <- Data$代替手段生成可否walk *exp(d1*Data$所要時間walk/100                                                     )

    ### 選択確率の計算
    ## 分母となる，各々のexp(V)の和をつくる
    deno <- (car + train + bus + bike + walk)

    ## それぞれ計算
    Ptrain <- Data$代替手段生成可否train*(train / deno)
    Pbus   <- Data$代替手段生成可否bus  *(bus   / deno)
    Pcar   <- Data$代替手段生成可否car  *(car   / deno)
    Pbike  <- Data$代替手段生成可否bike *(bike  / deno)
    Pwalk  <- Data$代替手段生成可否walk *(walk  / deno)
    
    ## 選択確率が0になってしまった場合に起こる問題の回避：logの真数条件によるエラーをケア
    Ptrain <- (Ptrain!=0)*Ptrain + (Ptrain==0)
    Pbus   <- (Pbus!=0)*Pbus     + (Pbus==0)
    Pcar   <- (Pcar  !=0)*Pcar   + (Pcar  ==0)
    Pbike  <- (Pbike  !=0)*Pbike + (Pbike  ==0)
    Pwalk  <- (Pwalk!=0)*Pwalk   + (Pwalk  ==0)
    
    ## 選択結果
    #代表交通手段が「鉄道」なら，Ctrainに1が入る
    Ctrain   <- Data$代表交通手段 =="鉄道"
    Cbus     <- Data$代表交通手段 =="バス"
    Ccar     <- Data$代表交通手段 =="自動車"
    Cbike    <- Data$代表交通手段 =="自転車"
    Cwalk    <- Data$代表交通手段 =="徒歩"
  
    ## 対数尤度の計算
    #simllが最終的な対数尤度
    LL <- colSums(Ctrain*log(Ptrain) + Cbus*log(Pbus) +
                Ccar  *log(Pcar)   + Cbike  *log(Pbike) +Cwalk *log(Pwalk))
    SimLL <- SimLL + LL 
  
  ##ループここまで 
  }

  ##計算の途中経過を表示
  print(x)
  print(SimLL/R)

  ##計算反復回数Rで割る
  SimLL <- SimLL / R

}

##### 対数尤度関数frの最大化#####

##パラメータ値の最適化 
#optim関数は最適化の関数
res <- maxLik(fr,start=b0,method= "BFGS")

## パラメータ推定値、ヘッセ行列 
b   <- res$estimate
hhh <- res$hessian

## t値の計算
tval <- b/sqrt(-diag(solve(hhh)))

## 初期尤度
L0 <- fr(b0)
## 最終尤度
LL <- res$maximum

##### 結果の出力 #####
print(res)
## 初期尤度
print(L0)
## 最終尤度
print(LL)
##ρ^2値
print((L0-LL)/L0)
## 修正済ρ^2値
print((L0-(LL-length(b)))/L0)
##パラメータ推定値 
print(b)
## t値 
print(tval)

